@@ -1,7 +1,7 @@
 /*
  * radius.c
  *
- * Copyright (C) 2012-19 - ntop.org
+ * Copyright (C) 2012-21 - ntop.org
  *
  * nDPI is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -40,11 +40,12 @@ static void ndpi_check_radius(struct ndpi_detection_module_struct *ndpi_struct,
   if(packet->udp != NULL) {
     struct radius_header *h = (struct radius_header*)packet->payload;
     /* RFC2865: The minimum length is 20 and maximum length is 4096. */
-    if((payload_len < 20) || (payload_len > 4096))
-	return;
-
-    if((payload_len > sizeof(struct radius_header))
-       && (h->code > 0)
+    if((payload_len < 20) || (payload_len > 4096)) {
+      NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
+      return;
+    }
+    
+    if((h->code > 0)
        && (h->code <= 13)
        && (ntohs(h->len) == payload_len)) {
       NDPI_LOG_INFO(ndpi_struct, "Found radius\n");
@@ -52,6 +53,7 @@ static void ndpi_check_radius(struct ndpi_detection_module_struct *ndpi_struct,
 
       return;
     }
+    
     NDPI_EXCLUDE_PROTO(ndpi_struct, flow);
     return;
   }
