@@ -1,7 +1,7 @@
 /*
  * skype.c
  *
- * Copyright (C) 2017-20 - ntop.org
+ * Copyright (C) 2017-21 - ntop.org
  *
  * nDPI is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,7 +19,7 @@
  */
 #include "ndpi_protocol_ids.h"
 
-#define NDPI_CURRENT_PROTO NDPI_PROTOCOL_SKYPE
+#define NDPI_CURRENT_PROTO NDPI_PROTOCOL_SKYPE_TEAMS
 
 #include "ndpi_api.h"
 
@@ -27,6 +27,45 @@ static int is_port(u_int16_t a, u_int16_t b, u_int16_t c) {
   return(((a == c) || (b == c)) ? 1 : 0);
 }
 
+static int ndpi_check_skype_udp_again(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow) {
+  struct ndpi_packet_struct *packet = &flow->packet;
+  u_int32_t payload_len = packet->payload_packet_len;
+
+  const uint8_t id_flags_iv_crc_len = 11;
+  const uint8_t crc_len = sizeof(flow->l4.udp.skype_crc);
+  const uint8_t crc_offset = id_flags_iv_crc_len - crc_len;
+
+  if (flow->packet_counter > 2)
+  {
+    /*
+     * Process only one packet after the initial packet received.
+     * This is required to prevent fals-positives with other protocols e.g. dnscrypt.
+     */
+    return 0;
+  }
+
+  if ((payload_len >= id_flags_iv_crc_len) && (packet->payload[2] == 0x02 /* Payload flag */ )) {
+    u_int8_t detected = 1;
+
+    /* Check if both packets have the same CRC */
+    for (int i = 0; i < crc_len && detected; i++) {
+      if (packet->payload[crc_offset + i] != flow->l4.udp.skype_crc[i])
+        detected = 0;
+    }
+
+    if (detected) {
+      ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE_TEAMS, NDPI_PROTOCOL_UNKNOWN);
+      flow->extra_packets_func = NULL;
+
+      /* Stop checking extra packets */
+      return 0;
+    }
+  }
+
+  /* Check more packets */
+  return 1;
+}
+
 static void ndpi_check_skype(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &flow->packet;
   // const u_int8_t *packet_payload = packet->payload;
@@ -70,9 +109,28 @@ static void ndpi_check_skype(struct ndpi_detection_module_struct *ndpi_struct, s
 	  if(is_port(sport, dport, 8801)) {
 	    ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_ZOOM, NDPI_PROTOCOL_UNKNOWN);
 	  } else if (payload_len >= 16 && packet->payload[0] != 0x01) /* Avoid invalid Cisco HSRP detection / RADIUS */ {
-	    ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE_CALL, NDPI_PROTOCOL_SKYPE);
+	    ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE_CALL, NDPI_PROTOCOL_SKYPE_TEAMS);
 	  }
 	}
+
+        if (flow->detected_protocol_stack[0] == NDPI_PROTOCOL_UNKNOWN) {
+          const uint8_t id_flags_iv_crc_len = 11;
+          const uint8_t crc_len = sizeof(flow->l4.udp.skype_crc);
+          const uint8_t crc_offset = id_flags_iv_crc_len - crc_len;
+
+          if ((payload_len >= id_flags_iv_crc_len)
+	      && (packet->payload[2] == 0x02 /* Payload flag */ )
+	      && (payload_len >= (crc_offset+crc_len))
+	      && (!flow->extra_packets_func)) {
+            flow->check_extra_packets = 1;
+            flow->max_extra_packets_to_check = 5;
+            flow->extra_packets_func = ndpi_check_skype_udp_again;
+
+            memcpy(flow->l4.udp.skype_crc, &packet->payload[crc_offset], crc_len);
+            return;
+          }
+        }
+
       }
       
       // return;
@@ -101,7 +159,7 @@ static void ndpi_check_skype(struct ndpi_detection_module_struct *ndpi_struct, s
 	/* printf("[SKYPE] %u/%u\n", ntohs(packet->tcp->source), ntohs(packet->tcp->dest)); */
 	
 	NDPI_LOG_INFO(ndpi_struct, "found skype\n");
-	  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE_CALL, NDPI_PROTOCOL_SKYPE);
+	  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE_TEAMS_CALL, NDPI_PROTOCOL_SKYPE_TEAMS);
       } else {
 	// printf("NO [SKYPE] payload_len=%u\n", payload_len);
       }
@@ -123,7 +181,7 @@ void ndpi_search_skype(struct ndpi_detection_module_struct *ndpi_struct, struct
   NDPI_LOG_DBG(ndpi_struct, "search skype\n");
 
   /* skip marked packets */
-  if(packet->detected_protocol_stack[0] != NDPI_PROTOCOL_SKYPE)
+  if(packet->detected_protocol_stack[0] != NDPI_PROTOCOL_SKYPE_TEAMS)
     ndpi_check_skype(ndpi_struct, flow);
 }
 
@@ -131,7 +189,7 @@ void ndpi_search_skype(struct ndpi_detection_module_struct *ndpi_struct, struct
 void init_skype_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask)
 {
   ndpi_set_bitmask_protocol_detection("Skype", ndpi_struct, detection_bitmask, *id,
-				      NDPI_PROTOCOL_SKYPE,
+				      NDPI_PROTOCOL_SKYPE_TEAMS,
 				      ndpi_search_skype,
 				      NDPI_SELECTION_BITMASK_PROTOCOL_TCP_OR_UDP_WITH_PAYLOAD,
 				      SAVE_DETECTION_BITMASK_AS_UNKNOWN,
