@@ -1,8 +1,8 @@
 /*
  * ssh.c
  *
- * Copyright (C) 2009-2011 by ipoque GmbH
- * Copyright (C) 2011-19 - ntop.org
+ * Copyright (C) 2011-21 - ntop.org
+ * Copyright (C) 2009-11 - ipoque GmbH
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -29,6 +29,8 @@
 #include "ndpi_api.h"
 #include "ndpi_md5.h"
 
+#include <string.h>
+
 /*
   HASSH - https://github.com/salesforce/hassh
 
@@ -54,21 +56,178 @@
   that usually is packet 14
 */
 
-/* #define SSH_DEBUG 1 */
+// #define SSH_DEBUG 1
+
+static void ndpi_search_ssh_tcp(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow);
+
+typedef struct {
+  const char *signature;
+  u_int16_t major, minor, patch;
+} ssh_pattern;
+
+/* ************************************************************************ */
+
+static void ssh_analyze_signature_version(struct ndpi_detection_module_struct *ndpi_struct,
+					  struct ndpi_flow_struct *flow,
+					  char *str_to_check,
+					  u_int8_t is_client_signature) {
+
+  if(str_to_check == NULL) return;
+  
+  u_int i;
+  u_int8_t obsolete_ssh_version = 0;  
+  const ssh_pattern ssh_servers_strings[] =
+    {
+     { (const char*)"SSH-%*f-OpenSSH_%d.%d.%d", 7, 0, 0 },     /* OpenSSH */
+     { (const char*)"SSH-%*f-APACHE-SSHD-%d.%d.%d", 2, 5, 1 }, /* Apache MINA SSHD */
+     { (const char*)"SSH-%*f-FileZilla_%d.%d.%d", 3, 40, 0 },  /* FileZilla SSH*/
+     { (const char*)"SSH-%*f-paramiko_%d.%d.%d", 2, 4, 0 },    /* Paramiko SSH */
+     { (const char*)"SSH-%*f-dropbear_%d.%d", 2020, 0, 0 },    /* Dropbear SSH */
+     { NULL, 0, 0, 0 } 
+    };
+
+  for(i = 0; ssh_servers_strings[i].signature != NULL; i++) {
+    int matches;
+    int major   = 0;
+    int minor   = 0;
+    int patch   = 0;
+    matches = sscanf(str_to_check, ssh_servers_strings[i].signature, &major, &minor, &patch);
+
+    if(matches == 3 || matches == 2) {
+      /* checking if is an old version */ 
+      if(major < ssh_servers_strings[i].major)
+	obsolete_ssh_version = 1;      
+      else if(major == ssh_servers_strings[i].major) {   
+	if(minor < ssh_servers_strings[i].minor)
+	  obsolete_ssh_version = 1;	
+	else if(minor == ssh_servers_strings[i].minor)
+	  if(patch < ssh_servers_strings[i].patch)
+	    obsolete_ssh_version = 1;
+      }
+
+#ifdef SSH_DEBUG
+      printf("[SSH] [SSH Version: %d.%d.%d]\n", major, minor, patch);
+#endif
+     
+      break;
+    }
+  }
+  
+  if(obsolete_ssh_version)
+    NDPI_SET_BIT(flow->risk,
+		 is_client_signature ? NDPI_SSH_OBSOLETE_CLIENT_VERSION_OR_CIPHER :
+		 NDPI_SSH_OBSOLETE_SERVER_VERSION_OR_CIPHER);
+}
+  
+/* ************************************************************************ */
+
+static void ssh_analyse_cipher(struct ndpi_detection_module_struct *ndpi_struct,
+			       struct ndpi_flow_struct *flow,
+			       char *ciphers, u_int cipher_len,
+			       u_int8_t is_client_signature) {
+
+  char *rem;
+  char *cipher;
+  u_int8_t found_obsolete_cipher = 0;
+  char *cipher_copy;
+  /*
+    List of obsolete ciphers can be found at
+    https://www.linuxminion.com/deprecated-ssh-cryptographic-settings/
+  */
+  const char *obsolete_ciphers[] = {
+				    "arcfour256",
+				    "arcfour128",
+				    "3des-cbc",
+				    "blowfish-cbc",
+				    "cast128-cbc",
+				    "arcfour",
+				    NULL, 
+  };
+
+  if((cipher_copy = (char*)ndpi_malloc(cipher_len+1)) == NULL) {
+#ifdef SSH_DEBUG
+    printf("[SSH] Nout enough memory\n");
+#endif
+    return;
+  }
+
+  strncpy(cipher_copy, ciphers, cipher_len);
+  cipher_copy[cipher_len] = '\0';
+
+  cipher = strtok_r(cipher_copy, ",", &rem);
+
+  while(cipher && !found_obsolete_cipher) {
+    u_int i;
+    
+    for(i = 0; obsolete_ciphers[i]; i++) {
+      if(strcmp(cipher, obsolete_ciphers[i]) == 0) {
+        found_obsolete_cipher = 1;
+#ifdef SSH_DEBUG
+	printf("[SSH] [SSH obsolete %s cipher][%s]\n",
+	       is_client_signature ? "client" : "server",
+	       obsolete_ciphers[i]);
+#endif   
+        break;
+      }
+    }
+
+    cipher = strtok_r(NULL, ",", &rem);
+  }
+
+  if(found_obsolete_cipher) {
+    ndpi_set_risk(ndpi_struct, flow,
+		  (is_client_signature ? NDPI_SSH_OBSOLETE_CLIENT_VERSION_OR_CIPHER : NDPI_SSH_OBSOLETE_SERVER_VERSION_OR_CIPHER));
+  }
+
+  ndpi_free(cipher_copy);
+}
+
+/* ************************************************************************ */
+
+static int search_ssh_again(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow) {
+  ndpi_search_ssh_tcp(ndpi_struct, flow);
+
+  if((flow->protos.ssh.hassh_client[0] != '\0')
+     && (flow->protos.ssh.hassh_server[0] != '\0')) {
+    /* stop extra processing */
+    flow->extra_packets_func = NULL; /* We're good now */
+    return(0);
+  }
+
+  /* Possibly more processing */
+  return(1);
+}
 
 /* ************************************************************************ */
 
 static void ndpi_int_ssh_add_connection(struct ndpi_detection_module_struct
 					*ndpi_struct, struct ndpi_flow_struct *flow) {
+  if(flow->extra_packets_func != NULL)
+    return;
+
+  flow->guessed_host_protocol_id = flow->guessed_protocol_id = NDPI_PROTOCOL_SSH;
+  
+  /* This is necessary to inform the core to call this dissector again */
+  flow->check_extra_packets = 1;
+  flow->max_extra_packets_to_check = 12;
+  flow->extra_packets_func = search_ssh_again;
+  
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SSH, NDPI_PROTOCOL_UNKNOWN);
 }
 
 /* ************************************************************************ */
 
-static u_int16_t concat_hash_string(struct ndpi_packet_struct *packet,
-				   char *buf, u_int8_t client_hash) {
-  u_int16_t offset = 22, buf_out_len = 0;
-  u_int32_t len = ntohl(*(u_int32_t*)&packet->payload[offset]);
+static u_int16_t concat_hash_string(struct ndpi_detection_module_struct *ndpi_struct,
+				    struct ndpi_flow_struct *flow,
+				    struct ndpi_packet_struct *packet,
+				    char *buf, u_int8_t client_hash) {
+  u_int32_t offset = 22, len, buf_out_len = 0, max_payload_len = packet->payload_packet_len-sizeof(u_int32_t);
+  const u_int32_t len_max = 65565;
+    
+  if(offset >= max_payload_len)
+    goto invalid_payload;
+
+  len = ntohl(*(u_int32_t*)&packet->payload[offset]);
   offset += 4;
 
   /* -1 for ';' */
@@ -80,118 +239,146 @@ static u_int16_t concat_hash_string(struct ndpi_packet_struct *packet,
   buf[buf_out_len++] = ';';
   offset += len;
 
+  if(offset >= max_payload_len)
+    goto invalid_payload;
+  
   /* ssh.server_host_key_algorithms [None] */
   len = ntohl(*(u_int32_t*)&packet->payload[offset]);
+
+  if(len > len_max)
+    goto invalid_payload;
   offset += 4 + len;
 
+  if(offset >= max_payload_len)
+    goto invalid_payload;
+
   /* ssh.encryption_algorithms_client_to_server [C] */
   len = ntohl(*(u_int32_t*)&packet->payload[offset]);
 
+  offset += 4;
   if(client_hash) {
-    offset += 4;
-
     if((offset >= packet->payload_packet_len) || (len >= packet->payload_packet_len-offset-1))
       goto invalid_payload;
 
     strncpy(&buf[buf_out_len], (const char *)&packet->payload[offset], len);
+    ssh_analyse_cipher(ndpi_struct, flow, (char*)&packet->payload[offset], len, 1 /* client */);
     buf_out_len += len;
     buf[buf_out_len++] = ';';
-    offset += len;
-  } else
-    offset += 4 + len;
+  }
+
+  if(len > len_max)
+    goto invalid_payload;
+  offset += len;
+
+  if(offset >= max_payload_len)
+    goto invalid_payload;
 
   /* ssh.encryption_algorithms_server_to_client [S] */
   len = ntohl(*(u_int32_t*)&packet->payload[offset]);
 
+  offset += 4;
   if(!client_hash) {
-    offset += 4;
-
     if((offset >= packet->payload_packet_len) || (len >= packet->payload_packet_len-offset-1))
       goto invalid_payload;
 
     strncpy(&buf[buf_out_len], (const char *)&packet->payload[offset], len);
+    ssh_analyse_cipher(ndpi_struct, flow, (char*)&packet->payload[offset], len, 0 /* server */);
     buf_out_len += len;
     buf[buf_out_len++] = ';';
-    offset += len;
-  } else
-    offset += 4 + len;
+  }
 
+  if(len > len_max)
+    goto invalid_payload;
+  offset += len;
+
+  if(offset >= max_payload_len)
+    goto invalid_payload;
   /* ssh.mac_algorithms_client_to_server [C] */
   len = ntohl(*(u_int32_t*)&packet->payload[offset]);
 
+  offset += 4;
   if(client_hash) {
-    offset += 4;
-
     if((offset >= packet->payload_packet_len) || (len >= packet->payload_packet_len-offset-1))
       goto invalid_payload;
 
     strncpy(&buf[buf_out_len], (const char *)&packet->payload[offset], len);
     buf_out_len += len;
     buf[buf_out_len++] = ';';
-    offset += len;
-  } else
-    offset += 4 + len;
+  }
+  
+  if(len > len_max)
+    goto invalid_payload;
+  offset += len;
 
+  if(offset >= max_payload_len)
+    goto invalid_payload;
   /* ssh.mac_algorithms_server_to_client [S] */
   len = ntohl(*(u_int32_t*)&packet->payload[offset]);
 
+  offset += 4;
   if(!client_hash) {
-    offset += 4;
-
     if((offset >= packet->payload_packet_len) || (len >= packet->payload_packet_len-offset-1))
       goto invalid_payload;
 
     strncpy(&buf[buf_out_len], (const char *)&packet->payload[offset], len);
     buf_out_len += len;
     buf[buf_out_len++] = ';';
-    offset += len;
-  } else
-    offset += 4 + len;
+  }
+
+  if(len > len_max)
+    goto invalid_payload;
+  offset += len;
 
   /* ssh.compression_algorithms_client_to_server [C] */
+  if(offset >= max_payload_len)
+    goto invalid_payload;
+  
   len = ntohl(*(u_int32_t*)&packet->payload[offset]);
 
+  offset += 4;
   if(client_hash) {
-    offset += 4;
-
     if((offset >= packet->payload_packet_len) || (len >= packet->payload_packet_len-offset-1))
       goto invalid_payload;
 
     strncpy(&buf[buf_out_len], (const char *)&packet->payload[offset], len);
     buf_out_len += len;
-    offset += len;
-  } else
-    offset += 4 + len;
+  }
 
+  if(len > len_max)
+    goto invalid_payload;
+  offset += len;
+
+  if(offset >= max_payload_len)
+    goto invalid_payload;
   /* ssh.compression_algorithms_server_to_client [S] */
   len = ntohl(*(u_int32_t*)&packet->payload[offset]);
 
+  offset += 4;
   if(!client_hash) {
-    offset += 4;
-
     if((offset >= packet->payload_packet_len) || (len >= packet->payload_packet_len-offset-1))
       goto invalid_payload;
 
     strncpy(&buf[buf_out_len], (const char *)&packet->payload[offset], len);
     buf_out_len += len;
-    offset += len;
-  } else
-    offset += 4 + len;
+  }
+
+  if(len > len_max)
+    goto invalid_payload;
+  offset += len;
 
   /* ssh.languages_client_to_server [None] */
 
   /* ssh.languages_server_to_client [None] */
 
 #ifdef SSH_DEBUG
-  printf("\n[SSH] %s\n", buf);
+  printf("[SSH] %s\n", buf);
 #endif
 
   return(buf_out_len);
 
-invalid_payload:
-
+ invalid_payload:
 #ifdef SSH_DEBUG
-  printf("\n[SSH] Invalid packet payload\n");
+  printf("[SSH] Invalid packet payload\n");
 #endif
 
   return(0);
@@ -213,78 +400,75 @@ static void ndpi_ssh_zap_cr(char *str, int len) {
 
 /* ************************************************************************ */
 
-void ndpi_search_ssh_tcp(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow) {
+static void ndpi_search_ssh_tcp(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow) {
   struct ndpi_packet_struct *packet = &flow->packet;
 
 #ifdef SSH_DEBUG
-  printf("\n[SSH] [stage: %u]\n", flow->l4.tcp.ssh_stage);
+  printf("[SSH] %s()\n", __FUNCTION__);
 #endif
 
   if(flow->l4.tcp.ssh_stage == 0) {
-    if(packet->payload_packet_len > 7 && packet->payload_packet_len < 100
-	&& memcmp(packet->payload, "SSH-", 4) == 0) {
-      if(!ndpi_struct->disable_metadata_export) {
-        int len = ndpi_min(sizeof(flow->protos.ssh.client_signature)-1, packet->payload_packet_len);
-
-	strncpy(flow->protos.ssh.client_signature, (const char *)packet->payload, len);
-	flow->protos.ssh.client_signature[len] = '\0';
-	ndpi_ssh_zap_cr(flow->protos.ssh.client_signature, len);
+    if(packet->payload_packet_len > 7
+       && memcmp(packet->payload, "SSH-", 4) == 0) {
+      int len = ndpi_min(sizeof(flow->protos.ssh.client_signature)-1, packet->payload_packet_len);
+      
+      strncpy(flow->protos.ssh.client_signature, (const char *)packet->payload, len);
+      flow->protos.ssh.client_signature[len] = '\0';
+      ndpi_ssh_zap_cr(flow->protos.ssh.client_signature, len);
 
+      ssh_analyze_signature_version(ndpi_struct, flow, flow->protos.ssh.client_signature, 1);
+      
 #ifdef SSH_DEBUG
-	printf("\n[SSH] [client_signature: %s]\n", flow->protos.ssh.client_signature);
-#endif
-      }
-
+      printf("[SSH] [client_signature: %s]\n", flow->protos.ssh.client_signature);
+#endif      
+      
       NDPI_LOG_DBG2(ndpi_struct, "ssh stage 0 passed\n");
       flow->l4.tcp.ssh_stage = 1 + packet->packet_direction;
-      flow->guessed_host_protocol_id = flow->guessed_protocol_id = NDPI_PROTOCOL_SSH;
+      ndpi_int_ssh_add_connection(ndpi_struct, flow);
       return;
     }
   } else if(flow->l4.tcp.ssh_stage == (2 - packet->packet_direction)) {
     if(packet->payload_packet_len > 7 && packet->payload_packet_len < 500
-	&& memcmp(packet->payload, "SSH-", 4) == 0) {
-      if(!ndpi_struct->disable_metadata_export) {
-	int len = ndpi_min(sizeof(flow->protos.ssh.server_signature)-1, packet->payload_packet_len);
-
-	strncpy(flow->protos.ssh.server_signature, (const char *)packet->payload, len);
-	flow->protos.ssh.server_signature[len] = '\0';
-	ndpi_ssh_zap_cr(flow->protos.ssh.server_signature, len);
+       && memcmp(packet->payload, "SSH-", 4) == 0) {
+      int len = ndpi_min(sizeof(flow->protos.ssh.server_signature)-1, packet->payload_packet_len);
+      
+      strncpy(flow->protos.ssh.server_signature, (const char *)packet->payload, len);
+      flow->protos.ssh.server_signature[len] = '\0';
+      ndpi_ssh_zap_cr(flow->protos.ssh.server_signature, len);
 
+      ssh_analyze_signature_version(ndpi_struct, flow, flow->protos.ssh.server_signature, 0);
+      
 #ifdef SSH_DEBUG
-	printf("\n[SSH] [server_signature: %s]\n", flow->protos.ssh.server_signature);
+      printf("[SSH] [server_signature: %s]\n", flow->protos.ssh.server_signature);
 #endif
-
-	NDPI_LOG_DBG2(ndpi_struct, "ssh stage 1 passed\n");
-	flow->guessed_host_protocol_id = flow->guessed_protocol_id = NDPI_PROTOCOL_SSH;
-      } else {
-	NDPI_LOG_INFO(ndpi_struct, "found ssh\n");
-	ndpi_int_ssh_add_connection(ndpi_struct, flow);
-      }
-
+      
+      NDPI_LOG_DBG2(ndpi_struct, "ssh stage 1 passed\n");
+      flow->guessed_host_protocol_id = flow->guessed_protocol_id = NDPI_PROTOCOL_SSH;
+      
 #ifdef SSH_DEBUG
-      printf("\n[SSH] [completed stage: %u]\n", flow->l4.tcp.ssh_stage);
+      printf("[SSH] [completed stage: %u]\n", flow->l4.tcp.ssh_stage);
 #endif
 
       flow->l4.tcp.ssh_stage = 3;
       return;
     }
-  } else {
+  } else if(packet->payload_packet_len > 5) {
     u_int8_t msgcode = *(packet->payload + 5);
     ndpi_MD5_CTX ctx;
-
-#ifdef SSH_DEBUG
-    printf("\n[SSH] [stage: %u][msg: %u]\n", flow->l4.tcp.ssh_stage, msgcode);
-#endif
-
+    
     if(msgcode == 20 /* key exchange init */) {
-      char *hassh_buf = calloc(packet->payload_packet_len, sizeof(char));
+      char *hassh_buf = ndpi_calloc(packet->payload_packet_len, sizeof(char));
       u_int i, len;
 
+#ifdef SSH_DEBUG
+      printf("[SSH] [stage: %u][msg: %u][direction: %u][key exchange init]\n", flow->l4.tcp.ssh_stage, msgcode, packet->packet_direction);
+#endif
+
       if(hassh_buf) {
-	if(flow->l4.tcp.ssh_stage == 3) {
+	if(packet->packet_direction == 0 /* client */) {
 	  u_char fingerprint_client[16];
 
-	  len = concat_hash_string(packet, hassh_buf, 1 /* client */);
+	  len = concat_hash_string(ndpi_struct, flow, packet, hassh_buf, 1 /* client */);
 
 	  ndpi_MD5Init(&ctx);
 	  ndpi_MD5Update(&ctx, (const unsigned char *)hassh_buf, len);
@@ -292,7 +476,7 @@ void ndpi_search_ssh_tcp(struct ndpi_detection_module_struct *ndpi_struct, struc
 
 #ifdef SSH_DEBUG
 	  {
-	    printf("\n[SSH] [client][%s][", hassh_buf);
+	    printf("[SSH] [client][%s][", hassh_buf);
 	    for(i=0; i<16; i++) printf("%02X", fingerprint_client[i]);
 	    printf("]\n");
 	  }
@@ -302,7 +486,7 @@ void ndpi_search_ssh_tcp(struct ndpi_detection_module_struct *ndpi_struct, struc
 	} else {
 	  u_char fingerprint_server[16];
 
-	  len = concat_hash_string(packet, hassh_buf, 0 /* server */);
+	  len = concat_hash_string(ndpi_struct, flow, packet, hassh_buf, 0 /* server */);
 
 	  ndpi_MD5Init(&ctx);
 	  ndpi_MD5Update(&ctx, (const unsigned char *)hassh_buf, len);
@@ -310,7 +494,7 @@ void ndpi_search_ssh_tcp(struct ndpi_detection_module_struct *ndpi_struct, struc
 
 #ifdef SSH_DEBUG
 	  {
-	    printf("\n[SSH] [server][%s][", hassh_buf);
+	    printf("[SSH] [server][%s][", hassh_buf);
 	    for(i=0; i<16; i++) printf("%02X", fingerprint_server[i]);
 	    printf("]\n");
 	  }
@@ -320,20 +504,24 @@ void ndpi_search_ssh_tcp(struct ndpi_detection_module_struct *ndpi_struct, struc
 	  flow->protos.ssh.hassh_server[32] = '\0';
 	}
 
-	free(hassh_buf);
+	ndpi_free(hassh_buf);
       }
+
+      ndpi_int_ssh_add_connection(ndpi_struct, flow);
     }
 
-    if(flow->l4.tcp.ssh_stage++ == 4) {
-      NDPI_LOG_INFO(ndpi_struct, "found ssh\n");
-      ndpi_int_ssh_add_connection(ndpi_struct, flow);
+    if((flow->protos.ssh.hassh_client[0] != '\0') && (flow->protos.ssh.hassh_server[0] != '\0')) {
+#ifdef SSH_DEBUG
+      printf("[SSH] Dissection completed\n");
+#endif
+      flow->extra_packets_func = NULL; /* We're good now */
     }
 
     return;
   }
 
 #ifdef SSH_DEBUG
-  printf("\n[SSH] Excluding SSH");
+  printf("[SSH] Excluding SSH");
 #endif
 
   NDPI_LOG_DBG(ndpi_struct, "excluding ssh at stage %d\n", flow->l4.tcp.ssh_stage);
