@@ -1,7 +1,7 @@
 /*
  * tcp_or_udp.c
  *
- * Copyright (C) 2011-20 - ntop.org
+ * Copyright (C) 2011-21 - ntop.org
  *
  * nDPI is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -73,11 +73,7 @@ void ndpi_search_tcp_or_udp(struct ndpi_detection_module_struct *ndpi_struct, st
     proto = ndpi_search_tcp_or_udp_raw(ndpi_struct,
 				       flow,
 				       flow->packet.iph ? flow->packet.iph->protocol :
-#ifdef NDPI_DETECTION_SUPPORT_IPV6
 				       flow->packet.iphv6->ip6_hdr.ip6_un1_nxt,
-#else
-				       0,
-#endif
 				       ntohl(packet->iph->saddr), 
 				       ntohl(packet->iph->daddr),
 				       sport, dport);
